@@ -5,13 +5,16 @@ DIE=0
 package=gst-plugins-base
 srcfile=gst/audiotestsrc/gstaudiotestsrc.c
 
-# Make sure we have common
-if test ! -f common/gst-autogen.sh;
+# a quick cvs co to ease the transition
+if test ! -d common;
 then
-  echo "+ Setting up common submodule"
-  git submodule init
+  echo "+ getting common/ from cvs"
+  if test -e CVS/Tag
+  then
+    TAG="-r `tail -c +2 CVS/Tag`"
+  fi
+  cvs co $TAG common
 fi
-git submodule update
 
 # source helper functions
 if test ! -f common/gst-autogen.sh;
@@ -22,33 +25,26 @@ then
 fi
 . common/gst-autogen.sh
 
-# install pre-commit hook for doing clean commits
-if test ! \( -x .git/hooks/pre-commit -a -L .git/hooks/pre-commit \);
-then
-    rm -f .git/hooks/pre-commit
-    ln -s ../../common/hooks/pre-commit.hook .git/hooks/pre-commit
-fi
-
-
 CONFIGURE_DEF_OPT='--enable-maintainer-mode --enable-gtk-doc'
 
 autogen_options $@
 
-printf "+ check for build tools"
+echo -n "+ check for build tools"
 if test ! -z "$NOCHECK"; then echo " skipped"; else  echo; fi
-version_check "autoconf" "$AUTOCONF autoconf autoconf270 autoconf269 autoconf268 autoconf267 autoconf266 autoconf265 autoconf264 autoconf263 autoconf262 autoconf261 autoconf260" \
-              "ftp://ftp.gnu.org/pub/gnu/autoconf/" 2 60 || DIE=1
-version_check "automake" "$AUTOMAKE automake automake-1.11 automake-1.10" \
-              "ftp://ftp.gnu.org/pub/gnu/automake/" 1 10 || DIE=1
+version_check "autoconf" "$AUTOCONF autoconf autoconf-2.54 autoconf-2.53 autoconf-2.52" \
+              "ftp://ftp.gnu.org/pub/gnu/autoconf/" 2 52 || DIE=1
+version_check "automake" "$AUTOMAKE automake automake-1.7 automake-1.6 automake-1.5" \
+              "ftp://ftp.gnu.org/pub/gnu/automake/" 1 6 || DIE=1
 version_check "autopoint" "autopoint" \
-              "ftp://ftp.gnu.org/pub/gnu/gettext/" 0 17 || DIE=1
-version_check "libtoolize" "$LIBTOOLIZE libtoolize glibtoolize" \
+              "ftp://ftp.gnu.org/pub/gnu/gettext/" 0 11 5 || DIE=1
+version_check "libtoolize" "$LIBTOOLIZE libtoolize" \
               "ftp://ftp.gnu.org/pub/gnu/libtool/" 1 5 0 || DIE=1
 version_check "pkg-config" "" \
               "http://www.freedesktop.org/software/pkgconfig" 0 8 0 || DIE=1
 
 die_check $DIE
 
+autoconf_2_52d_check || DIE=1
 aclocal_check || DIE=1
 autoheader_check || DIE=1
 
@@ -76,11 +72,11 @@ fi
 tool_run "$autopoint --force"
 patch -p0 < common/gettext.patch
 
-tool_run "$libtoolize" "--copy --force"
 tool_run "$aclocal" "-I m4 -I common/m4 $ACLOCAL_FLAGS"
+tool_run "$libtoolize" "--copy --force"
 tool_run "$autoheader"
 
-# touch the stamp-h.in build stamp so we don't re-run autoheader in maintainer mode
+# touch the stamp-h.in build stamp so we don't re-run autoheader in maintainer mode -- wingo
 echo timestamp > stamp-h.in 2> /dev/null
 
 tool_run "$autoconf"
