@@ -36,9 +36,6 @@
 #include <ogg/ogg.h>
 #include <string.h>
 
-#include "gstogg.h"
-#include "gstoggstream.h"
-
 GST_DEBUG_CATEGORY_STATIC (gst_ogg_parse_debug);
 #define GST_CAT_DEFAULT gst_ogg_parse_debug
 
@@ -46,13 +43,29 @@ GST_DEBUG_CATEGORY_STATIC (gst_ogg_parse_debug);
 #define GST_OGG_PARSE(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_OGG_PARSE, GstOggParse))
 #define GST_OGG_PARSE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_OGG_PARSE, GstOggParse))
 #define GST_IS_OGG_PARSE(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_OGG_PARSE))
-#define GST_IS_OGG_PARSE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_OGG_PARSE))
+#define GST_IS_OGG_PARSE_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_OGG_PARSE))
 
 static GType gst_ogg_parse_get_type (void);
 
 typedef struct _GstOggParse GstOggParse;
 typedef struct _GstOggParseClass GstOggParseClass;
 
+/* Each ogg logical stream has a GstOggStream associated with it */
+typedef struct
+{
+  /*ogg_stream_state stream; *//* We need this to get the packets out in order
+     to do codec identification, for various
+     codec-specific tasks */
+
+  gboolean in_headers;          /* Initially true, false once we've read all the
+                                   headers for this logical stream */
+
+  guint32 serialno;             /* Unique serial number of this stream */
+
+  GSList *headers;              /* List of ogg pages that we'll set on caps */
+  GSList *unknown_pages;        /* List of pages we haven't yet classified */
+} GstOggStream;
+
 struct _GstOggParse
 {
   GstElement element;
@@ -70,10 +83,6 @@ struct _GstOggParse
   gboolean last_page_not_bos;   /* Set if we've seen a non-BOS page */
 
   ogg_sync_state sync;          /* Ogg page synchronisation */
-
-  GstCaps *caps;                /* Our src caps */
-
-  GstOggStream *video_stream;   /* Stream used to construct delta_unit flags */
 };
 
 struct _GstOggParseClass
@@ -113,11 +122,10 @@ gst_ogg_parse_get_type (void)
 static void
 free_stream (GstOggStream * stream)
 {
-  g_list_foreach (stream->headers, (GFunc) gst_mini_object_unref, NULL);
-  g_list_foreach (stream->unknown_pages, (GFunc) gst_mini_object_unref, NULL);
-  g_list_foreach (stream->stored_buffers, (GFunc) gst_mini_object_unref, NULL);
+  g_slist_foreach (stream->headers, (GFunc) gst_mini_object_unref, NULL);
+  g_slist_foreach (stream->unknown_pages, (GFunc) gst_mini_object_unref, NULL);
 
-  g_slice_free (GstOggStream, stream);
+  g_free (stream);
 }
 
 static void
@@ -129,52 +137,37 @@ gst_ogg_parse_delete_all_streams (GstOggParse * ogg)
 }
 
 static GstOggStream *
-gst_ogg_parse_new_stream (GstOggParse * parser, ogg_page * page)
+gst_ogg_parse_new_stream (GstOggParse * parser, guint32 serialno)
 {
-  GstOggStream *stream;
-  ogg_packet packet;
-  int ret;
-  guint32 serialno;
+  GstOggStream *ret;
 
-  serialno = ogg_page_serialno (page);
+  GST_DEBUG_OBJECT (parser, "creating new stream %08lx", serialno);
 
-  GST_DEBUG_OBJECT (parser, "creating new stream %08x", serialno);
+  ret = g_new0 (GstOggStream, 1);
 
-  stream = g_slice_new0 (GstOggStream);
+  ret->serialno = serialno;
+  ret->in_headers = 1;
 
-  stream->serialno = serialno;
-  stream->in_headers = 1;
+  /*
+     if (ogg_stream_init (&ret->stream, serialno) != 0) {
+     GST_ERROR ("Could not initialize ogg_stream struct for serial %08lx.",
+     serialno);
+     return NULL;
+     }
+   */
 
-  if (ogg_stream_init (&stream->stream, serialno) != 0) {
-    GST_ERROR ("Could not initialize ogg_stream struct for serial %08x.",
-        serialno);
-    return NULL;
-  }
+  parser->oggstreams = g_slist_append (parser->oggstreams, ret);
 
-  /* FIXME check return */
-  ogg_stream_pagein (&stream->stream, page);
-
-  /* FIXME check return */
-  ret = ogg_stream_packetout (&stream->stream, &packet);
-  if (ret == 1) {
-    gst_ogg_stream_setup_map (stream, &packet);
-    if (stream->is_video) {
-      parser->video_stream = stream;
-    }
-  }
-
-  parser->oggstreams = g_slist_append (parser->oggstreams, stream);
-
-  return stream;
+  return ret;
 }
 
 static GstOggStream *
 gst_ogg_parse_find_stream (GstOggParse * parser, guint32 serialno)
 {
-  GSList *l;
+  gint i;
 
-  for (l = parser->oggstreams; l != NULL; l = l->next) {
-    GstOggStream *stream = (GstOggStream *) l->data;
+  for (i = 0; i < g_slist_length (parser->oggstreams); i++) {
+    GstOggStream *stream = g_slist_nth_data (parser->oggstreams, i);
 
     if (stream->serialno == serialno)
       return stream;
@@ -218,16 +211,18 @@ static void
 gst_ogg_parse_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-
-  gst_element_class_set_details_simple (element_class,
-      "Ogg parser", "Codec/Parser",
+  static GstElementDetails gst_ogg_parse_details =
+      GST_ELEMENT_DETAILS ("ogg parser",
+      "Codec/Parser",
       "parse ogg streams into pages (info about ogg: http://xiph.org)",
       "Michael Smith <msmith@fluendo.com>");
 
-  gst_element_class_add_static_pad_template (element_class,
-      &ogg_parse_sink_template_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &ogg_parse_src_template_factory);
+  gst_element_class_set_details (element_class, &gst_ogg_parse_details);
+
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&ogg_parse_sink_template_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&ogg_parse_src_template_factory));
 }
 
 static void
@@ -236,7 +231,7 @@ gst_ogg_parse_class_init (GstOggParseClass * klass)
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  parent_class = g_type_class_peek_parent (klass);
+  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
   gstelement_class->change_state = gst_ogg_parse_change_state;
 
@@ -273,63 +268,44 @@ gst_ogg_parse_dispose (GObject * object)
   ogg_sync_clear (&ogg->sync);
   gst_ogg_parse_delete_all_streams (ogg);
 
-  if (ogg->caps) {
-    gst_caps_unref (ogg->caps);
-    ogg->caps = NULL;
-  }
-
   if (G_OBJECT_CLASS (parent_class)->dispose)
     G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
-/* submit the given buffer to the ogg sync */
-static GstFlowReturn
+/* submit the given buffer to the ogg sync.
+ *
+ * Returns the number of bytes submited.
+ */
+static gint
 gst_ogg_parse_submit_buffer (GstOggParse * ogg, GstBuffer * buffer)
 {
   guint size;
   guint8 *data;
   gchar *oggbuffer;
-  GstFlowReturn ret = GST_FLOW_OK;
 
   size = GST_BUFFER_SIZE (buffer);
   data = GST_BUFFER_DATA (buffer);
 
-  GST_DEBUG_OBJECT (ogg, "submitting %u bytes", size);
-  if (G_UNLIKELY (size == 0))
-    goto done;
-
+  /* We now have a buffer, submit it to the ogg sync layer */
   oggbuffer = ogg_sync_buffer (&ogg->sync, size);
-  if (G_UNLIKELY (oggbuffer == NULL)) {
-    GST_ELEMENT_ERROR (ogg, STREAM, DECODE,
-        (NULL), ("failed to get ogg sync buffer"));
-    ret = GST_FLOW_ERROR;
-    goto done;
-  }
-
   memcpy (oggbuffer, data, size);
-  if (G_UNLIKELY (ogg_sync_wrote (&ogg->sync, size) < 0)) {
-    GST_ELEMENT_ERROR (ogg, STREAM, DECODE,
-        (NULL), ("failed to write %d bytes to the sync buffer", size));
-    ret = GST_FLOW_ERROR;
-  }
+  ogg_sync_wrote (&ogg->sync, size);
 
-done:
+  /* We've copied all the neccesary data, so we're done with the buffer */
   gst_buffer_unref (buffer);
 
-  return ret;
+  return size;
 }
 
 static void
 gst_ogg_parse_append_header (GValue * array, GstBuffer * buf)
 {
   GValue value = { 0 };
-  /* We require a copy to avoid circular refcounts */
-  GstBuffer *buffer = gst_buffer_copy (buf);
 
   GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_IN_CAPS);
 
   g_value_init (&value, GST_TYPE_BUFFER);
-  gst_value_set_buffer (&value, buffer);
+  gst_value_set_buffer (&value, buf);
   gst_value_array_append_value (array, &value);
   g_value_unset (&value);
 
@@ -363,7 +339,7 @@ gst_ogg_parse_is_header (GstOggParse * ogg, GstOggStream * stream,
 
 static GstBuffer *
 gst_ogg_parse_buffer_from_page (ogg_page * page,
-    guint64 offset, GstClockTime timestamp)
+    guint64 offset, gboolean delta, GstClockTime timestamp)
 {
   int size = page->header_len + page->body_len;
   GstBuffer *buf = gst_buffer_new_and_alloc (size);
@@ -415,47 +391,25 @@ gst_ogg_parse_chain (GstPad * pad, GstBuffer * buffer)
       ogg->offset -= ret;
     } else {
       gint64 granule = ogg_page_granulepos (&page);
-#ifndef GST_DISABLE_GST_DEBUG
       int bos = ogg_page_bos (&page);
-#endif
       guint64 startoffset = ogg->offset;
-      GstOggStream *stream;
-      gboolean keyframe;
-
-      serialno = ogg_page_serialno (&page);
-      stream = gst_ogg_parse_find_stream (ogg, serialno);
 
       GST_LOG_OBJECT (ogg, "Timestamping outgoing buffer as %" GST_TIME_FORMAT,
           GST_TIME_ARGS (buffertimestamp));
-
-      if (stream) {
-        buffertimestamp = gst_ogg_stream_get_end_time_for_granulepos (stream,
-            granule);
-        if (ogg->video_stream) {
-          if (stream == ogg->video_stream) {
-            keyframe = gst_ogg_stream_granulepos_is_key_frame (stream, granule);
-          } else {
-            keyframe = FALSE;
-          }
-        } else {
-          keyframe = TRUE;
-        }
-      } else {
-        buffertimestamp = GST_CLOCK_TIME_NONE;
-        keyframe = TRUE;
-      }
-      pagebuffer = gst_ogg_parse_buffer_from_page (&page, startoffset,
+      /* Turn our page into a GstBuffer TODO: better timestamps? Requires format
+       * parsing. */
+      pagebuffer = gst_ogg_parse_buffer_from_page (&page, startoffset, FALSE,
           buffertimestamp);
 
       /* We read out 'ret' bytes, so we set the next offset appropriately */
       ogg->offset += ret;
 
+      serialno = ogg_page_serialno (&page);
+
       GST_LOG_OBJECT (ogg,
-          "processing ogg page (serial %08x, pageno %ld, "
-          "granule pos %" G_GUINT64_FORMAT ", bos %d, offset %"
-          G_GUINT64_FORMAT "-%" G_GUINT64_FORMAT ") keyframe=%d",
-          serialno, ogg_page_pageno (&page),
-          granule, bos, startoffset, ogg->offset, keyframe);
+          "processing ogg page (serial %08lx, pageno %ld, granule pos %llu, "
+          "bos %d, offset %llu-%llu)", serialno, ogg_page_pageno (&page),
+          granule, bos, startoffset, ogg->offset);
 
       if (ogg_page_bos (&page)) {
         /* If we've seen this serialno before, this is technically an error,
@@ -466,8 +420,8 @@ gst_ogg_parse_chain (GstPad * pad, GstBuffer * buffer)
         GstOggStream *stream = gst_ogg_parse_find_stream (ogg, serialno);
 
         if (stream != NULL) {
-          GST_LOG_OBJECT (ogg, "Incorrect stream; repeats serial number %08x "
-              "at offset %" G_GINT64_FORMAT, serialno, ogg->offset);
+          GST_LOG_OBJECT (ogg, "Incorrect stream; repeats serial number %u "
+              "at offset %lld", serialno, ogg->offset);
         }
 
         if (ogg->last_page_not_bos) {
@@ -476,16 +430,15 @@ gst_ogg_parse_chain (GstPad * pad, GstBuffer * buffer)
           gst_ogg_parse_delete_all_streams (ogg);
         }
 
-        stream = gst_ogg_parse_new_stream (ogg, &page);
+        stream = gst_ogg_parse_new_stream (ogg, serialno);
 
         ogg->last_page_not_bos = FALSE;
 
         gst_buffer_ref (pagebuffer);
-        stream->headers = g_list_append (stream->headers, pagebuffer);
+        stream->headers = g_slist_append (stream->headers, pagebuffer);
 
         if (!ogg->in_headers) {
-          GST_LOG_OBJECT (ogg,
-              "Found start of new chain at offset %" G_GUINT64_FORMAT,
+          GST_LOG_OBJECT (ogg, "Found start of new chain at offset %llu",
               startoffset);
           ogg->in_headers = 1;
         }
@@ -507,8 +460,7 @@ gst_ogg_parse_chain (GstPad * pad, GstBuffer * buffer)
         GstOggStream *stream = gst_ogg_parse_find_stream (ogg, serialno);
 
         if (!stream) {
-          GST_LOG_OBJECT (ogg,
-              "Non-BOS page unexpectedly found at %" G_GINT64_FORMAT,
+          GST_LOG_OBJECT (ogg, "Non-BOS page unexpectedly found at %lld",
               ogg->offset);
           goto failure;
         }
@@ -520,22 +472,22 @@ gst_ogg_parse_chain (GstPad * pad, GstBuffer * buffer)
         if (type == PAGE_PENDING && ogg->in_headers) {
           gst_buffer_ref (pagebuffer);
 
-          stream->unknown_pages = g_list_append (stream->unknown_pages,
+          stream->unknown_pages = g_slist_append (stream->unknown_pages,
               pagebuffer);
         } else if (type == PAGE_HEADER) {
           if (!ogg->in_headers) {
             GST_LOG_OBJECT (ogg, "Header page unexpectedly found outside "
-                "headers at offset %" G_GINT64_FORMAT, ogg->offset);
+                "headers at offset %lld", ogg->offset);
             goto failure;
           } else {
             /* Append the header to the buffer list, after any unknown previous
              * pages
              */
-            stream->headers = g_list_concat (stream->headers,
+            stream->headers = g_slist_concat (stream->headers,
                 stream->unknown_pages);
-            g_list_free (stream->unknown_pages);
+            g_slist_free (stream->unknown_pages);
             gst_buffer_ref (pagebuffer);
-            stream->headers = g_list_append (stream->headers, pagebuffer);
+            stream->headers = g_slist_append (stream->headers, pagebuffer);
           }
         } else {                /* PAGE_DATA, or PAGE_PENDING but outside headers */
           if (ogg->in_headers) {
@@ -550,17 +502,16 @@ gst_ogg_parse_chain (GstPad * pad, GstBuffer * buffer)
             GstCaps *caps;
             GstStructure *structure;
             GValue array = { 0 };
-            gint count = 0;
+            gint i, count = 0;
             gboolean found_pending_headers = FALSE;
-            GSList *l;
 
             g_value_init (&array, GST_TYPE_ARRAY);
 
-            for (l = ogg->oggstreams; l != NULL; l = l->next) {
-              GstOggStream *stream = (GstOggStream *) l->data;
+            for (i = 0; i < g_slist_length (ogg->oggstreams); i++) {
+              GstOggStream *stream = g_slist_nth_data (ogg->oggstreams, i);
 
-              if (g_list_length (stream->headers) == 0) {
-                GST_LOG_OBJECT (ogg, "No primary header found for stream %08x",
+              if (g_slist_length (stream->headers) == 0) {
+                GST_LOG_OBJECT (ogg, "No primary header found for stream %u",
                     stream->serialno);
                 goto failure;
               }
@@ -570,13 +521,13 @@ gst_ogg_parse_chain (GstPad * pad, GstBuffer * buffer)
               count++;
             }
 
-            for (l = ogg->oggstreams; l != NULL; l = l->next) {
-              GstOggStream *stream = (GstOggStream *) l->data;
-              GList *j;
+            for (i = 0; i < g_slist_length (ogg->oggstreams); i++) {
+              GstOggStream *stream = g_slist_nth_data (ogg->oggstreams, i);
+              int j;
 
-              /* already appended the first header, now do headers 2-N */
-              for (j = stream->headers->next; j != NULL; j = j->next) {
-                gst_ogg_parse_append_header (&array, GST_BUFFER (j->data));
+              for (j = 1; j < g_slist_length (stream->headers); j++) {
+                gst_ogg_parse_append_header (&array,
+                    GST_BUFFER (g_slist_nth_data (stream->headers, j)));
                 count++;
               }
             }
@@ -591,37 +542,27 @@ gst_ogg_parse_chain (GstPad * pad, GstBuffer * buffer)
 
             g_value_unset (&array);
 
-            if (ogg->caps)
-              gst_caps_unref (ogg->caps);
-            ogg->caps = caps;
+//            gst_caps_free(caps);
 
             GST_LOG_OBJECT (ogg, "Set \"streamheader\" caps with %d buffers "
                 "(one per page)", count);
 
             /* Now, we do the same thing, but push buffers... */
-            for (l = ogg->oggstreams; l != NULL; l = l->next) {
-              GstOggStream *stream = (GstOggStream *) l->data;
-              GstBuffer *buf = GST_BUFFER (stream->headers->data);
+            for (i = 0; i < g_slist_length (ogg->oggstreams); i++) {
+              GstOggStream *stream = g_slist_nth_data (ogg->oggstreams, i);
 
-              buf = gst_buffer_make_metadata_writable (buf);
-              gst_buffer_set_caps (buf, caps);
-
-              result = gst_pad_push (ogg->srcpad, buf);
+              result = gst_pad_push (ogg->srcpad,
+                  GST_BUFFER (stream->headers->data));
               if (result != GST_FLOW_OK)
                 return result;
             }
-            for (l = ogg->oggstreams; l != NULL; l = l->next) {
-              GstOggStream *stream = (GstOggStream *) l->data;
-              GList *j;
+            for (i = 0; i < g_slist_length (ogg->oggstreams); i++) {
+              GstOggStream *stream = g_slist_nth_data (ogg->oggstreams, i);
+              int j;
 
-              /* pushed the first one for each stream already, now do 2-N */
-              for (j = stream->headers->next; j != NULL; j = j->next) {
-                GstBuffer *buf = GST_BUFFER (j->data);
-
-                buf = gst_buffer_make_metadata_writable (buf);
-                gst_buffer_set_caps (buf, caps);
-
-                result = gst_pad_push (ogg->srcpad, buf);
+              for (j = 1; j < g_slist_length (stream->headers); j++) {
+                result = gst_pad_push (ogg->srcpad,
+                    GST_BUFFER (g_slist_nth_data (stream->headers, j)));
                 if (result != GST_FLOW_OK)
                   return result;
               }
@@ -630,72 +571,39 @@ gst_ogg_parse_chain (GstPad * pad, GstBuffer * buffer)
             ogg->in_headers = 0;
 
             /* And finally the pending data pages */
-            for (l = ogg->oggstreams; l != NULL; l = l->next) {
-              GstOggStream *stream = (GstOggStream *) l->data;
-              GList *k;
+            for (i = 0; i < g_slist_length (ogg->oggstreams); i++) {
+              GstOggStream *stream = g_slist_nth_data (ogg->oggstreams, i);
+              int j;
 
-              if (stream->unknown_pages == NULL)
+              if (g_slist_length (stream->unknown_pages) > 0) {
+                if (found_pending_headers) {
+                  GST_WARNING_OBJECT (ogg, "Incorrectly muxed headers found at "
+                      "approximate offset %lld", ogg->offset);
+                }
+                found_pending_headers = TRUE;
+              } else
                 continue;
 
-              if (found_pending_headers) {
-                GST_WARNING_OBJECT (ogg, "Incorrectly muxed headers found at "
-                    "approximate offset %" G_GINT64_FORMAT, ogg->offset);
-              }
-              found_pending_headers = TRUE;
-
               GST_LOG_OBJECT (ogg, "Pushing %d pending pages after headers",
-                  g_list_length (stream->unknown_pages) + 1);
-
-              for (k = stream->unknown_pages; k != NULL; k = k->next) {
-                GstBuffer *buf;
-
-                buf = gst_buffer_make_metadata_writable (GST_BUFFER (k->data));
-                gst_buffer_set_caps (buf, caps);
-                result = gst_pad_push (ogg->srcpad, buf);
+                  g_slist_length (stream->unknown_pages) + 1);
+              for (j = 0; j < g_slist_length (stream->unknown_pages); j++) {
+                result = gst_pad_push (ogg->srcpad,
+                    GST_BUFFER (g_slist_nth_data (stream->unknown_pages, j)));
                 if (result != GST_FLOW_OK)
                   return result;
               }
-              g_list_foreach (stream->unknown_pages,
+              g_slist_foreach (stream->unknown_pages,
                   (GFunc) gst_mini_object_unref, NULL);
-              g_list_free (stream->unknown_pages);
+              g_slist_free (stream->unknown_pages);
               stream->unknown_pages = NULL;
             }
-          }
 
-          if (granule == -1) {
-            stream->stored_buffers = g_list_append (stream->stored_buffers,
-                pagebuffer);
+            result = gst_pad_push (ogg->srcpad, GST_BUFFER (pagebuffer));
+            if (result != GST_FLOW_OK)
+              return result;
           } else {
-            while (stream->stored_buffers) {
-              GstBuffer *buf = stream->stored_buffers->data;
-
-              buf = gst_buffer_make_metadata_writable (buf);
-              gst_buffer_set_caps (buf, ogg->caps);
-              GST_BUFFER_TIMESTAMP (buf) = buffertimestamp;
-              if (!keyframe) {
-                GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DELTA_UNIT);
-              } else {
-                keyframe = FALSE;
-              }
-
-              result = gst_pad_push (ogg->srcpad, buf);
-              if (result != GST_FLOW_OK)
-                return result;
-
-              stream->stored_buffers =
-                  g_list_delete_link (stream->stored_buffers,
-                  stream->stored_buffers);
-            }
-
-            pagebuffer = gst_buffer_make_metadata_writable (pagebuffer);
-            gst_buffer_set_caps (pagebuffer, ogg->caps);
-            if (!keyframe) {
-              GST_BUFFER_FLAG_SET (pagebuffer, GST_BUFFER_FLAG_DELTA_UNIT);
-            } else {
-              keyframe = FALSE;
-            }
-
-            result = gst_pad_push (ogg->srcpad, pagebuffer);
+            /* Normal data page, submit buffer */
+            result = gst_pad_push (ogg->srcpad, GST_BUFFER (pagebuffer));
             if (result != GST_FLOW_OK)
               return result;
           }
