@@ -22,23 +22,30 @@
  * SECTION:element-clockoverlay
  * @see_also: #GstTextOverlay, #GstTimeOverlay
  *
+ * <refsect2>
+ * <para>
  * This element overlays the current clock time on top of a video
  * stream. You can position the text and configure the font details
  * using the properties of the #GstTextOverlay class. By default, the
  * time is displayed in the top left corner of the picture, with some
  * padding to the left and to the top.
- *
- * <refsect2>
- * <title>Example launch lines</title>
- * |[
+ * </para>
+ * <para>
+ * Here is a simple pipeline that displays the current time in the top left
+ * corner of the video picture:
+ * <programlisting>
  * gst-launch -v videotestsrc ! clockoverlay ! xvimagesink
- * ]| Display the current time in the top left corner of the video picture
- * |[
- * gst-launch -v videotestsrc ! clockoverlay halign=right valign=bottom text="Edge City" shaded-background=true ! ffmpegcolorspace ! ximagesink
- * ]| Another pipeline that displays the current time with some leading
+ * </programlisting>
+ * </para>
+ * <para>
+ * Here is another pipeline that displays the current time with some leading
  * text in the bottom right corner of the video picture, with the background
  * of the text being shaded in order to make it more legible on top of a
- * bright video background.
+ * bright video background:
+ * <programlisting>
+ * gst-launch -v videotestsrc ! clockoverlay halign=right valign=bottom text="Edge City" shaded-background=true ! ffmpegcolorspace ! ximagesink
+ * </programlisting>
+ * </para>
  * </refsect2>
  */
 
@@ -50,78 +57,47 @@
 #include <gst/video/video.h>
 #include <time.h>
 
-
-#define DEFAULT_PROP_TIMEFORMAT 	"%H:%M:%S"
-
-enum
-{
-  PROP_0,
-  PROP_TIMEFORMAT,
-  PROP_LAST
-};
+static GstElementDetails clock_overlay_details =
+GST_ELEMENT_DETAILS ("Clock Overlay",
+    "Filter/Editor/Video",
+    "Overlays the current clock time on a video stream",
+    "Tim-Philipp Müller <tim@centricular.net>");
 
 GST_BOILERPLATE (GstClockOverlay, gst_clock_overlay, GstTextOverlay,
-    GST_TYPE_TEXT_OVERLAY);
+    GST_TYPE_TEXT_OVERLAY)
 
-static void
-gst_clock_overlay_base_init (gpointer g_class)
+     static void gst_clock_overlay_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details_simple (element_class, "Clock overlay",
-      "Filter/Editor/Video",
-      "Overlays the current clock time on a video stream",
-      "Tim-Philipp Müller <tim@centricular.net>");
+  gst_element_class_set_details (element_class, &clock_overlay_details);
 }
 
-
-static void gst_clock_overlay_finalize (GObject * object);
-static void gst_clock_overlay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec);
-static void gst_clock_overlay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec);
-
 static gchar *
 gst_clock_overlay_render_time (GstClockOverlay * overlay)
 {
-  struct tm *t;
+  struct tm t;
   time_t now;
-  gchar buf[256];
-
-#ifdef HAVE_LOCALTIME_R
-  struct tm dummy;
-#endif
 
   now = time (NULL);
-
-#ifdef HAVE_LOCALTIME_R
-  /* Need to call tzset explicitly when calling localtime_r for changes
-     to the timezone between calls to be visible.  */
-  tzset ();
-  t = localtime_r (&now, &dummy);
-#else
-  /* on win32 this apparently returns a per-thread struct which would be fine */
-  t = localtime (&now);
-#endif
-
-  if (t == NULL)
+  if (localtime_r (&now, &t) == NULL)
     return g_strdup ("--:--:--");
 
-  if (strftime (buf, sizeof (buf), overlay->format, t) == 0)
-    return g_strdup ("");
-  return g_strdup (buf);
+  return g_strdup_printf ("%02u:%02u:%02u", t.tm_hour, t.tm_min, t.tm_sec);
 }
 
-/* Called with lock held */
 static gchar *
 gst_clock_overlay_get_text (GstTextOverlay * overlay, GstBuffer * video_frame)
 {
   gchar *time_str, *txt, *ret;
-  GstClockOverlay *clock_overlay = GST_CLOCK_OVERLAY (overlay);
 
+  overlay->need_render = TRUE;
+
+  GST_OBJECT_LOCK (overlay);
   txt = g_strdup (overlay->default_text);
+  GST_OBJECT_UNLOCK (overlay);
 
-  time_str = gst_clock_overlay_render_time (clock_overlay);
+  time_str = gst_clock_overlay_render_time (GST_CLOCK_OVERLAY (overlay));
   if (txt != NULL && *txt != '\0') {
     ret = g_strdup_printf ("%s %s", txt, time_str);
   } else {
@@ -129,12 +105,6 @@ gst_clock_overlay_get_text (GstTextOverlay * overlay, GstBuffer * video_frame)
     time_str = NULL;
   }
 
-  if (g_strcmp0 (ret, clock_overlay->text)) {
-    overlay->need_render = TRUE;
-    g_free (clock_overlay->text);
-    clock_overlay->text = g_strdup (ret);
-  }
-
   g_free (txt);
   g_free (time_str);
 
@@ -144,33 +114,29 @@ gst_clock_overlay_get_text (GstTextOverlay * overlay, GstBuffer * video_frame)
 static void
 gst_clock_overlay_class_init (GstClockOverlayClass * klass)
 {
-  GObjectClass *gobject_class;
   GstTextOverlayClass *gsttextoverlay_class;
-  PangoContext *context;
-  PangoFontDescription *font_description;
 
-  gobject_class = (GObjectClass *) klass;
   gsttextoverlay_class = (GstTextOverlayClass *) klass;
 
-  gobject_class->finalize = gst_clock_overlay_finalize;
-  gobject_class->set_property = gst_clock_overlay_set_property;
-  gobject_class->get_property = gst_clock_overlay_get_property;
-
   gsttextoverlay_class->get_text = gst_clock_overlay_get_text;
+}
 
-  g_object_class_install_property (gobject_class, PROP_TIMEFORMAT,
-      g_param_spec_string ("time-format", "Date/Time Format",
-          "Format to use for time and date value, as in strftime.",
-          DEFAULT_PROP_TIMEFORMAT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+static void
+gst_clock_overlay_init (GstClockOverlay * overlay, GstClockOverlayClass * klass)
+{
+  PangoFontDescription *font_description;
+  GstTextOverlay *textoverlay;
+  PangoContext *context;
+
+  textoverlay = GST_TEXT_OVERLAY (overlay);
 
-  g_mutex_lock (GST_TEXT_OVERLAY_CLASS (klass)->pango_lock);
   context = GST_TEXT_OVERLAY_CLASS (klass)->pango_context;
 
   pango_context_set_language (context, pango_language_from_string ("en_US"));
   pango_context_set_base_dir (context, PANGO_DIRECTION_LTR);
 
   font_description = pango_font_description_new ();
-  pango_font_description_set_family_static (font_description, "Monospace");
+  pango_font_description_set_family (font_description, g_strdup ("Monospace"));
   pango_font_description_set_style (font_description, PANGO_STYLE_NORMAL);
   pango_font_description_set_variant (font_description, PANGO_VARIANT_NORMAL);
   pango_font_description_set_weight (font_description, PANGO_WEIGHT_NORMAL);
@@ -178,71 +144,7 @@ gst_clock_overlay_class_init (GstClockOverlayClass * klass)
   pango_font_description_set_size (font_description, 18 * PANGO_SCALE);
   pango_context_set_font_description (context, font_description);
   pango_font_description_free (font_description);
-  g_mutex_unlock (GST_TEXT_OVERLAY_CLASS (klass)->pango_lock);
-}
-
-
-static void
-gst_clock_overlay_finalize (GObject * object)
-{
-  GstClockOverlay *overlay = GST_CLOCK_OVERLAY (object);
-
-  g_free (overlay->format);
-  g_free (overlay->text);
-  overlay->format = NULL;
-
-  G_OBJECT_CLASS (parent_class)->finalize (object);
-}
-
-
-static void
-gst_clock_overlay_init (GstClockOverlay * overlay, GstClockOverlayClass * klass)
-{
-  GstTextOverlay *textoverlay;
-
-  textoverlay = GST_TEXT_OVERLAY (overlay);
 
   textoverlay->valign = GST_TEXT_OVERLAY_VALIGN_TOP;
   textoverlay->halign = GST_TEXT_OVERLAY_HALIGN_LEFT;
-
-  overlay->format = g_strdup (DEFAULT_PROP_TIMEFORMAT);
-}
-
-
-static void
-gst_clock_overlay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstClockOverlay *overlay = GST_CLOCK_OVERLAY (object);
-
-  GST_OBJECT_LOCK (overlay);
-  switch (prop_id) {
-    case PROP_TIMEFORMAT:
-      g_free (overlay->format);
-      overlay->format = g_value_dup_string (value);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-  GST_OBJECT_UNLOCK (overlay);
-}
-
-
-static void
-gst_clock_overlay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstClockOverlay *overlay = GST_CLOCK_OVERLAY (object);
-
-  GST_OBJECT_LOCK (overlay);
-  switch (prop_id) {
-    case PROP_TIMEFORMAT:
-      g_value_set_string (value, overlay->format);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-  GST_OBJECT_UNLOCK (overlay);
 }
