@@ -38,46 +38,12 @@
 /**
  * SECTION:gstcddabasesrc
  * @short_description: Base class for CD digital audio (CDDA) sources
- *
+ * 
  * <refsect2>
  * <para>
  * Provides a base class for CDDA sources, which handles things like seeking,
  * querying, discid calculation, tags, and buffer timestamping.
  * </para>
- * <title>Using GstCddaBaseSrc-based elements in applications</title>
- * <para>
- * GstCddaBaseSrc registers two #GstFormat<!-- -->s of its own, namely
- * the "track" format and the "sector" format. Applications will usually
- * only find the "track" format interesting. You can retrieve that #GstFormat
- * for use in seek events or queries with gst_format_get_by_nick("track").
- * </para>
- * <para>
- * In order to query the number of tracks, for example, an application would
- * set the CDDA source element to READY or PAUSED state and then query the
- * the number of tracks via gst_element_query_duration() using the track
- * format acquired above. Applications can query the currently playing track
- * in the same way.
- * </para>
- * <para>
- * Alternatively, applications may retrieve the currently playing track and
- * the total number of tracks from the taglist that will posted on the bus
- * whenever the CD is opened or the currently playing track changes. The
- * taglist will contain GST_TAG_TRACK_NUMBER and GST_TAG_TRACK_COUNT tags.
- * </para>
- * <para>
- * Applications playing back CD audio using playbin and cdda://n URIs should
- * issue a seek command in track format to change between tracks, rather than
- * setting a new cdda://n+1 URI on playbin (as setting a new URI on playbin
- * involves closing and re-opening the CD device, which is much much slower).
- * </para>
- * <title>Tags and meta-information</title>
- * <para>
- * CDDA sources will automatically emit a number of tags, details about which
- * can be found in the libgsttag documentation. Those tags are:
- * #GST_TAG_CDDA_CDDB_DISCID, #GST_TAG_CDDA_CDDB_DISCID_FULL,
- * #GST_TAG_CDDA_MUSICBRAINZ_DISCID, #GST_TAG_CDDA_MUSICBRAINZ_DISCID_FULL,
- * among others.
- * </para>
  * </refsect2>
  */
 
@@ -86,7 +52,6 @@
 #endif
 
 #include <string.h>
-#include <stdlib.h>             /* for strtol */
 
 #include "gstcddabasesrc.h"
 #include "gst/gst-i18n-plugin.h"
@@ -104,6 +69,8 @@ GST_DEBUG_CATEGORY_STATIC (gst_cdda_base_src_debug);
 #define TIME_INTERVAL_FROM_SECTORS(sectors)  ((SAMPLES_PER_SECTOR * sectors * GST_SECOND) / 44100)
 #define SECTORS_FROM_TIME_INTERVAL(dtime)    (dtime * 44100 / (SAMPLES_PER_SECTOR * GST_SECOND))
 
+#define GST_TYPE_CDDA_BASE_SRC_MODE          (gst_cdda_base_src_mode_get_type ())
+
 enum
 {
   ARG_0,
@@ -158,11 +125,11 @@ GST_STATIC_PAD_TEMPLATE ("src",
 static GstFormat track_format;
 static GstFormat sector_format;
 
-GType
+static GType
 gst_cdda_base_src_mode_get_type (void)
 {
   static GType mode_type;       /* 0 */
-  static const GEnumValue modes[] = {
+  static GEnumValue modes[] = {
     {GST_CDDA_BASE_SRC_MODE_NORMAL, "Stream consists of a single track",
         "normal"},
     {GST_CDDA_BASE_SRC_MODE_CONTINUOUS, "Stream consists of the whole disc",
@@ -181,15 +148,30 @@ gst_cdda_base_src_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_cdda_base_src_src_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_cdda_base_src_src_template));
 
   /* our very own formats */
   track_format = gst_format_register ("track", "CD track");
   sector_format = gst_format_register ("sector", "CD sector");
 
-  /* register CDDA tags */
-  gst_tag_register_musicbrainz_tags ();
+  /* tags */
+  gst_tag_register (GST_TAG_CDDA_CDDB_DISCID, GST_TAG_FLAG_META,
+      G_TYPE_STRING, "discid", "CDDB discid for metadata retrieval",
+      gst_tag_merge_use_first);
+
+  gst_tag_register (GST_TAG_CDDA_CDDB_DISCID_FULL, GST_TAG_FLAG_META,
+      G_TYPE_STRING, "discid full",
+      "CDDB discid for metadata retrieval (full)", gst_tag_merge_use_first);
+
+  gst_tag_register (GST_TAG_CDDA_MUSICBRAINZ_DISCID, GST_TAG_FLAG_META,
+      G_TYPE_STRING, "musicbrainz-discid",
+      "Musicbrainz discid for metadata retrieval", gst_tag_merge_use_first);
+
+  gst_tag_register (GST_TAG_CDDA_MUSICBRAINZ_DISCID_FULL, GST_TAG_FLAG_META,
+      G_TYPE_STRING, "musicbrainz-discid-full",
+      "Musicbrainz discid for metadata retrieval (full)",
+      gst_tag_merge_use_first);
 
 #if 0
   ///// FIXME: what type to use here? ///////
@@ -219,15 +201,14 @@ gst_cdda_base_src_class_init (GstCddaBaseSrcClass * klass)
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_DEVICE,
       g_param_spec_string ("device", "Device", "CD device location",
-          NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          NULL, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MODE,
       g_param_spec_enum ("mode", "Mode", "Mode", GST_TYPE_CDDA_BASE_SRC_MODE,
-          GST_CDDA_BASE_SRC_MODE_NORMAL,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          GST_CDDA_BASE_SRC_MODE_NORMAL, G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_TRACK,
       g_param_spec_uint ("track", "Track", "Track", 1, 99, 1,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
 
 #if 0
   /* Do we really need this toc adjustment stuff as properties? does the user
@@ -238,13 +219,12 @@ gst_cdda_base_src_class_init (GstCddaBaseSrcClass * klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_TOC_OFFSET,
       g_param_spec_int ("toc-offset", "Table of contents offset",
           "Add <n> sectors to the values reported", G_MININT, G_MAXINT, 0,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_TOC_BIAS,
       g_param_spec_boolean ("toc-bias", "Table of contents bias",
           "Assume that the beginning offset of track 1 as reported in the TOC "
           "will be addressed as LBA 0.  Necessary for some Toshiba drives to "
-          "get track boundaries", FALSE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          "get track boundaries", FALSE, G_PARAM_READWRITE));
 #endif
 
   element_class->set_index = GST_DEBUG_FUNCPTR (gst_cdda_base_src_set_index);
@@ -284,44 +264,9 @@ gst_cdda_base_src_finalize (GObject * obj)
   g_free (cddasrc->uri);
   g_free (cddasrc->device);
 
-  if (cddasrc->index)
-    gst_object_unref (cddasrc->index);
-
   G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
-static void
-gst_cdda_base_src_set_device (GstCddaBaseSrc * src, const gchar * device)
-{
-  if (src->device)
-    g_free (src->device);
-  src->device = NULL;
-
-  if (!device)
-    return;
-
-  /* skip multiple slashes */
-  while (*device == '/' && *(device + 1) == '/')
-    device++;
-
-#ifdef __sun
-  /*
-   * On Solaris, /dev/rdsk is used for accessing the CD device, but some
-   * applications pass in /dev/dsk, so correct.
-   */
-  if (strncmp (device, "/dev/dsk", 8) == 0) {
-    gchar *rdsk_value;
-    rdsk_value = g_strdup_printf ("/dev/rdsk%s", device + 8);
-    src->device = g_strdup (rdsk_value);
-    g_free (rdsk_value);
-  } else {
-#endif
-    src->device = g_strdup (device);
-#ifdef __sun
-  }
-#endif
-}
-
 static void
 gst_cdda_base_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -338,7 +283,12 @@ gst_cdda_base_src_set_property (GObject * object, guint prop_id,
     case ARG_DEVICE:{
       const gchar *dev = g_value_get_string (value);
 
-      gst_cdda_base_src_set_device (src, dev);
+      g_free (src->device);
+      if (dev && *dev) {
+        src->device = g_strdup (dev);
+      } else {
+        src->device = NULL;
+      }
       break;
     }
     case ARG_TRACK:{
@@ -694,8 +644,8 @@ gst_cdda_base_src_query (GstBaseSrc * basesrc, GstQuery * query)
       break;
     }
     default:{
-      GST_DEBUG_OBJECT (src, "unhandled query, chaining up to parent class");
-      return GST_BASE_SRC_CLASS (parent_class)->query (basesrc, query);
+      GST_DEBUG_OBJECT (src, "unsupported query type");
+      return FALSE;
     }
   }
 
@@ -734,7 +684,7 @@ gst_cdda_base_src_do_seek (GstBaseSrc * basesrc, GstSegment * segment)
       seek_sector += src->tracks[0].start;
       break;
     default:
-      g_return_val_if_reached (FALSE);
+      g_assert_not_reached ();
   }
 
   src->cur_sector = (gint) seek_sector;
@@ -784,7 +734,7 @@ gst_cdda_base_src_handle_track_seek (GstCddaBaseSrc * src, gdouble rate,
         start_time = -1;
         break;
       default:
-        g_return_val_if_reached (FALSE);
+        g_assert_not_reached ();
     }
 
     switch (stop_type) {
@@ -809,7 +759,7 @@ gst_cdda_base_src_handle_track_seek (GstCddaBaseSrc * src, gdouble rate,
         stop_time = -1;
         break;
       default:
-        g_return_val_if_reached (FALSE);
+        g_assert_not_reached ();
     }
 
     GST_LOG_OBJECT (src, "seek segment %" GST_TIME_FORMAT "-%" GST_TIME_FORMAT,
@@ -840,7 +790,7 @@ gst_cdda_base_src_handle_track_seek (GstCddaBaseSrc * src, gdouble rate,
     return FALSE;
   }
 
-  GST_DEBUG_OBJECT (src, "seeking to track %" G_GINT64_FORMAT, start + 1);
+  GST_DEBUG_OBJECT (src, "seeking to track %d", start + 1);
 
   src->cur_sector = src->tracks[start].start;
   GST_DEBUG_OBJECT (src, "starting at sector %d", src->cur_sector);
@@ -898,13 +848,14 @@ gst_cdda_base_src_handle_event (GstBaseSrc * basesrc, GstEvent * event)
       } else {
         GST_LOG_OBJECT (src, "let base class handle seek in %s format",
             gst_format_get_name (format));
-        event = gst_event_ref (event);
+        gst_event_ref (event);
         ret = GST_BASE_SRC_CLASS (parent_class)->event (basesrc, event);
       }
       break;
     }
     default:{
       GST_LOG_OBJECT (src, "let base class handle event");
+      gst_event_ref (event);
       ret = GST_BASE_SRC_CLASS (parent_class)->event (basesrc, event);
       break;
     }
@@ -913,7 +864,7 @@ gst_cdda_base_src_handle_event (GstBaseSrc * basesrc, GstEvent * event)
   return ret;
 }
 
-static GstURIType
+static guint
 gst_cdda_base_src_uri_get_type (void)
 {
   return GST_URI_SRC;
@@ -922,7 +873,7 @@ gst_cdda_base_src_uri_get_type (void)
 static gchar **
 gst_cdda_base_src_uri_get_protocols (void)
 {
-  static gchar *protocols[] = { (char *) "cdda", NULL };
+  static gchar *protocols[] = { "cdda", NULL };
 
   return protocols;
 }
@@ -937,9 +888,7 @@ gst_cdda_base_src_uri_get_uri (GstURIHandler * handler)
   g_free (src->uri);
 
   if (GST_OBJECT_FLAG_IS_SET (GST_BASE_SRC (src), GST_BASE_SRC_STARTED)) {
-    src->uri =
-        g_strdup_printf ("cdda://%s#%d", src->device,
-        (src->uri_track > 0) ? src->uri_track : 1);
+    src->uri = g_strdup_printf ("cdda://%d", src->uri_track);
   } else {
     src->uri = g_strdup ("cdda://1");
   }
@@ -952,49 +901,31 @@ gst_cdda_base_src_uri_get_uri (GstURIHandler * handler)
 /* Note: gst_element_make_from_uri() might call us with just 'cdda://' as
  * URI and expects us to return TRUE then (and this might be in any state) */
 
-/* We accept URIs of the format cdda://(device#track)|(track) */
-
 static gboolean
 gst_cdda_base_src_uri_set_uri (GstURIHandler * handler, const gchar * uri)
 {
   GstCddaBaseSrc *src = GST_CDDA_BASE_SRC (handler);
-  gchar *protocol;
-  const gchar *location;
-  gchar *track_number;
+  gchar *protocol, *location;
 
   GST_OBJECT_LOCK (src);
 
   protocol = gst_uri_get_protocol (uri);
-  if (!protocol || g_ascii_strcasecmp (protocol, "cdda") != 0) {
+  if (!protocol || strcmp (protocol, "cdda") != 0) {
     g_free (protocol);
     goto failed;
   }
   g_free (protocol);
 
-  location = uri + 7;
-  track_number = g_strrstr (location, "#");
-  src->uri_track = 0;
-  /* FIXME 0.11: ignore URI fragments that look like device paths for
-   * the benefit of rhythmbox and possibly other applications.
-   */
-  if (track_number && track_number[1] != '/') {
-    gchar *device, *nuri = g_strdup (uri);
-
-    track_number = nuri + (track_number - uri);
-    *track_number = '\0';
-    device = gst_uri_get_location (nuri);
-    gst_cdda_base_src_set_device (src, device);
-    g_free (device);
-    src->uri_track = strtol (track_number + 1, NULL, 10);
-    g_free (nuri);
-  } else {
-    if (*location == '\0')
-      src->uri_track = 1;
-    else
-      src->uri_track = strtol (location, NULL, 10);
+  location = gst_uri_get_location (uri);
+  if (location == NULL || *location == '\0') {
+    g_free (location);
+    location = g_strdup ("1");
   }
 
-  if (src->uri_track < 1)
+  src->uri_track = strtol (location, NULL, 10);
+  g_free (location);
+
+  if (src->uri_track == 0)
     goto failed;
 
   if (src->num_tracks > 0
@@ -1074,10 +1005,7 @@ gst_cdda_base_src_add_track (GstCddaBaseSrc * src, GstCddaBaseSrcTrack * track)
   if (src->num_tracks > 0) {
     guint end_of_previous_track = src->tracks[src->num_tracks - 1].end;
 
-    if (track->start <= end_of_previous_track) {
-      GST_WARNING ("track %2u overlaps with previous tracks", track->num);
-      return FALSE;
-    }
+    g_return_val_if_fail (track->start >= end_of_previous_track, FALSE);
   }
 
   GST_OBJECT_LOCK (src);
@@ -1131,35 +1059,36 @@ cddb_sum (gint n)
   return ret;
 }
 
+#include "base64.h"
+#include "sha1.h"
+
 static void
 gst_cddabasesrc_calculate_musicbrainz_discid (GstCddaBaseSrc * src)
 {
   GString *s;
-  GChecksum *sha;
-  guchar digest[20];
-  gchar *ptr;
+  SHA_INFO sha;
+  guchar digest[20], *ptr;
   gchar tmp[9];
   gulong i;
   guint leadout_sector;
-  gsize digest_len;
 
   s = g_string_new (NULL);
 
   leadout_sector = src->tracks[src->num_tracks - 1].end + 1 + CD_MSF_OFFSET;
 
   /* generate SHA digest */
-  sha = g_checksum_new (G_CHECKSUM_SHA1);
+  sha_init (&sha);
   g_snprintf (tmp, sizeof (tmp), "%02X", src->tracks[0].num);
   g_string_append_printf (s, "%02X", src->tracks[0].num);
-  g_checksum_update (sha, (guchar *) tmp, 2);
+  sha_update (&sha, (SHA_BYTE *) tmp, 2);
 
   g_snprintf (tmp, sizeof (tmp), "%02X", src->tracks[src->num_tracks - 1].num);
   g_string_append_printf (s, " %02X", src->tracks[src->num_tracks - 1].num);
-  g_checksum_update (sha, (guchar *) tmp, 2);
+  sha_update (&sha, (SHA_BYTE *) tmp, 2);
 
   g_snprintf (tmp, sizeof (tmp), "%08X", leadout_sector);
   g_string_append_printf (s, " %08X", leadout_sector);
-  g_checksum_update (sha, (guchar *) tmp, 8);
+  sha_update (&sha, (SHA_BYTE *) tmp, 8);
 
   for (i = 0; i < 99; i++) {
     if (i < src->num_tracks) {
@@ -1167,36 +1096,21 @@ gst_cddabasesrc_calculate_musicbrainz_discid (GstCddaBaseSrc * src)
 
       g_snprintf (tmp, sizeof (tmp), "%08X", frame_offset);
       g_string_append_printf (s, " %08X", frame_offset);
-      g_checksum_update (sha, (guchar *) tmp, 8);
+      sha_update (&sha, (SHA_BYTE *) tmp, 8);
     } else {
-      g_checksum_update (sha, (guchar *) "00000000", 8);
+      sha_update (&sha, (SHA_BYTE *) "00000000", 8);
     }
   }
-  digest_len = 20;
-  g_checksum_get_digest (sha, (guint8 *) & digest, &digest_len);
+  sha_final (digest, &sha);
 
   /* re-encode to base64 */
-  ptr = g_base64_encode (digest, digest_len);
-  g_checksum_free (sha);
-  i = strlen (ptr);
+  ptr = rfc822_binary (digest, 20, &i);
 
   g_assert (i < sizeof (src->mb_discid) + 1);
   memcpy (src->mb_discid, ptr, i);
   src->mb_discid[i] = '\0';
   free (ptr);
 
-  /* Replace '/', '+' and '=' by '_', '.' and '-' as specified on
-   * http://musicbrainz.org/doc/DiscIDCalculation
-   */
-  for (ptr = src->mb_discid; *ptr != '\0'; ptr++) {
-    if (*ptr == '/')
-      *ptr = '_';
-    else if (*ptr == '+')
-      *ptr = '.';
-    else if (*ptr == '=')
-      *ptr = '-';
-  }
-
   GST_DEBUG_OBJECT (src, "musicbrainz-discid      = %s", src->mb_discid);
   GST_DEBUG_OBJECT (src, "musicbrainz-discid-full = %s", s->str);
 
@@ -1322,7 +1236,7 @@ gst_cdda_base_src_add_tags (GstCddaBaseSrc * src)
         GST_FORMAT_TIME, &duration);
 
     gst_tag_list_add (src->tracks[i].tags,
-        GST_TAG_MERGE_REPLACE,
+        GST_TAG_MERGE_REPLACE_ALL,
         GST_TAG_TRACK_NUMBER, i + 1,
         GST_TAG_TRACK_COUNT, src->num_tracks, GST_TAG_DURATION, duration, NULL);
   }
@@ -1336,7 +1250,7 @@ gst_cdda_base_src_add_tags (GstCddaBaseSrc * src)
    * gst_tag_list_get_value_index() rather than use tag names incl.
    * the track number ?? *////////////////////////////////////////
 
-  gst_tag_list_add (src->tags, GST_TAG_MERGE_REPLACE,
+  gst_tag_list_add (src->tags, GST_TAG_MERGE_REPLACE_ALL,
       GST_TAG_TRACK_COUNT, src->num_tracks, NULL);
 #if 0
   for (i = 0; i < src->num_tracks; ++i) {
@@ -1370,26 +1284,12 @@ static void
 gst_cdda_base_src_set_index (GstElement * element, GstIndex * index)
 {
   GstCddaBaseSrc *src = GST_CDDA_BASE_SRC (element);
-  GstIndex *old;
 
-  GST_OBJECT_LOCK (element);
-  old = src->index;
-  if (old == index) {
-    GST_OBJECT_UNLOCK (element);
-    return;
-  }
-  if (index)
-    gst_object_ref (index);
   src->index = index;
-  GST_OBJECT_UNLOCK (element);
-  if (old)
-    gst_object_unref (old);
 
-  if (index) {
-    gst_index_get_writer_id (index, GST_OBJECT (src), &src->index_id);
-    gst_index_add_format (index, src->index_id, track_format);
-    gst_index_add_format (index, src->index_id, sector_format);
-  }
+  gst_index_get_writer_id (index, GST_OBJECT (src), &src->index_id);
+  gst_index_add_format (index, src->index_id, track_format);
+  gst_index_add_format (index, src->index_id, sector_format);
 }
 
 
@@ -1397,14 +1297,8 @@ static GstIndex *
 gst_cdda_base_src_get_index (GstElement * element)
 {
   GstCddaBaseSrc *src = GST_CDDA_BASE_SRC (element);
-  GstIndex *index;
 
-  GST_OBJECT_LOCK (element);
-  if ((index = src->index))
-    gst_object_ref (index);
-  GST_OBJECT_UNLOCK (element);
-
-  return index;
+  return src->index;
 }
 
 static gint
@@ -1567,7 +1461,6 @@ gst_cdda_base_src_stop (GstBaseSrc * basesrc)
   return TRUE;
 }
 
-
 static GstFlowReturn
 gst_cdda_base_src_create (GstPushSrc * pushsrc, GstBuffer ** buffer)
 {
@@ -1576,31 +1469,29 @@ gst_cdda_base_src_create (GstPushSrc * pushsrc, GstBuffer ** buffer)
   GstBuffer *buf;
   GstFormat format;
   gboolean eos;
-
-  GstClockTime position = GST_CLOCK_TIME_NONE;
-  GstClockTime duration = GST_CLOCK_TIME_NONE;
-  gint64 qry_position;
+  gint64 position;
+  gint64 duration;
 
   g_assert (klass->read_sector != NULL);
 
   switch (src->mode) {
     case GST_CDDA_BASE_SRC_MODE_NORMAL:
-      eos = (src->cur_sector > src->tracks[src->cur_track].end);
+      eos = (src->cur_sector >= src->tracks[src->cur_track].end);
       break;
     case GST_CDDA_BASE_SRC_MODE_CONTINUOUS:
-      eos = (src->cur_sector > src->tracks[src->num_tracks - 1].end);
+      eos = (src->cur_sector >= src->tracks[src->num_tracks - 1].end);
       src->cur_track = gst_cdda_base_src_get_track_from_sector (src,
           src->cur_sector);
       break;
     default:
-      g_return_val_if_reached (GST_FLOW_ERROR);
+      g_assert_not_reached ();
   }
 
   if (eos) {
     src->prev_track = -1;
     GST_DEBUG_OBJECT (src, "EOS at sector %d, cur_track=%d, mode=%d",
         src->cur_sector, src->cur_track, src->mode);
-    /* base class will send EOS for us */
+    gst_pad_push_event (GST_BASE_SRC_PAD (src), gst_event_new_eos ());
     return GST_FLOW_UNEXPECTED;
   }
 
@@ -1633,23 +1524,13 @@ gst_cdda_base_src_create (GstPushSrc * pushsrc, GstBuffer ** buffer)
   }
 
   format = GST_FORMAT_TIME;
-  if (gst_pad_query_position (GST_BASE_SRC_PAD (src), &format, &qry_position)) {
-    gint64 next_ts = 0;
-
-    position = (GstClockTime) qry_position;
-
-    ++src->cur_sector;
-    if (gst_pad_query_position (GST_BASE_SRC_PAD (src), &format, &next_ts)) {
-      duration = (GstClockTime) (next_ts - qry_position);
-    }
-    --src->cur_sector;
+  if (!gst_pad_query_position (GST_BASE_SRC_PAD (src), &format, &position)) {
+    position = GST_CLOCK_TIME_NONE;
   }
 
-  /* fallback duration: 4 bytes per sample, 44100 samples per second */
-  if (duration == GST_CLOCK_TIME_NONE) {
-    duration = gst_util_uint64_scale_int (GST_BUFFER_SIZE (buf) >> 2,
-        GST_SECOND, 44100);
-  }
+  /* 4 bytes per sample, 44100 samples per second */
+  duration = gst_util_uint64_scale_int (GST_BUFFER_SIZE (buf) >> 2, GST_SECOND,
+      44100);
 
   GST_BUFFER_TIMESTAMP (buf) = position;
   GST_BUFFER_DURATION (buf) = duration;
